// ATtiny84/85 IR Transmitter
// Transmits fixed NEC format IR commands
// Must know the specific codes being transmitted
// Message length: 32 data bits, 1 stop bit
//
//NOTE: The ATtiny84/85 must be set to use the 8-MHz internal clock.
//NOTE: Must disable millis/micros in the config bits when using PWM.
//      The "delayMicroseconds" routine is still usable but not "delay".
//
// Written by Boomer48: February 2021

#include <avr/sleep.h>

// Comment this line out for ATtiny84
//#define Tiny85

#define NEC_HDR_MARK  9000
#define NEC_HDR_SPACE 4500
#define NEC_BIT_MARK  560
#define NEC_ONE_SPACE 1600
#define NEC_ZERO_SPACE  560

#define F_CPU 8000000UL

#ifdef Tiny85
#define DD_Reg DDRB // Data Direction Register for physical pin 6 PWM
#define DD_Bit DDB1 // Data Direction Bit for physical pin 6 PWM
#else
#define DD_Reg DDRB // Data Direction Register for physical pin 5 PWM
#define DD_Bit DDB2 // Data Direction Bit for physical pin 5 PWM
#endif

#ifdef Tiny85
#define LED 0
#define Button1 2
#define Button2 3
#define Button3 4
#else
#define LED 10
#define Button1 0
#define Button2 1
#define Button3 2
#define Button4 3
#define Button5 4
#define Button6 5
#define Button7 6
#define Button8 7
#endif

void setup ()
{
  digitalWrite (Button1, HIGH);  // enable pull-up
  digitalWrite (Button2, HIGH);  // enable pull-up
  digitalWrite (Button3, HIGH);  // enable pull-up
#ifndef Tiny85
  digitalWrite (Button4, HIGH);  // enable pull-up
  digitalWrite (Button5, HIGH);  // enable pull-up
  digitalWrite (Button6, HIGH);  // enable pull-up
  digitalWrite (Button7, HIGH);  // enable pull-up
  digitalWrite (Button8, HIGH);  // enable pull-up
#endif
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

  EnableIROut();

  // Set interrupt-on-change registers for buttons
  bitClear(GIMSK, INT0); // disable INT0 external interrupt

#ifdef Tiny85
  bitSet(GIMSK, PCIE); // enable interrupt-on-change
  bitSet(PCMSK, Button1);
  bitSet(PCMSK, Button2);
  bitSet(PCMSK, Button3);
#else
  bitSet(GIMSK, PCIE0); // enable interrupt-on-change (PCINT0-PCINT5)
  bitSet(PCMSK0, Button1);
  bitSet(PCMSK0, Button2);
  bitSet(PCMSK0, Button3);
  bitSet(PCMSK0, Button4);
  bitSet(PCMSK0, Button5);
  bitSet(PCMSK0, Button6);
  bitSet(PCMSK0, Button7);
  bitSet(PCMSK0, Button8);
#endif

  bitClear(ADCSRA, ADEN); // disable ADC for low power mode
  set_sleep_mode (SLEEP_MODE_PWR_DOWN);

}  // end of setup

void loop ()
{
  sleep_enable();

  // Do not interrupt before we go to sleep, or the
  // ISR will detach interrupts and we won't wake.
  noInterrupts();

  // We are guaranteed that the sleep_cpu call will be done
  // as the processor executes the next instruction after
  // interrupts are turned on.
  interrupts();  // one cycle
  sleep_cpu();   // one cycle

} // end of loop

//--------------------------------------------
void Write_IR(byte Cmd)
{
  byte temp;

  PWM_On(NEC_HDR_MARK);
  PWM_Off(NEC_HDR_SPACE);

  for (int i = 0; i < 8; i++)
  {
    // Send Header (always 0x00 0xFF)
    PWM_On(NEC_BIT_MARK);
    PWM_Off(NEC_ZERO_SPACE);
  }

  for (int i = 0; i < 8; i++)
  {
    // Send Header (always 0x00 0xFF)
    PWM_On(NEC_BIT_MARK);
    PWM_Off(NEC_ONE_SPACE);
  }

  temp = Cmd;
  for (int i = 0; i < 8; i++) {
    // Send Command, then send opposite
    if (bitRead(temp, 0) == 1)
    {
      PWM_On(NEC_BIT_MARK);
      PWM_Off(NEC_ONE_SPACE);
    }
    else
    {
      PWM_On(NEC_BIT_MARK);
      PWM_Off(NEC_ZERO_SPACE);
    }
    temp >>= 1;
  }

  temp = Cmd;
  for (int i = 0; i < 8; i++) {
    // Send Command, then send opposite
    if (bitRead(temp, 0) == 1)
    {
      PWM_On(NEC_BIT_MARK);
      PWM_Off(NEC_ZERO_SPACE);
    }
    else
    {
      PWM_On(NEC_BIT_MARK);
      PWM_Off(NEC_ONE_SPACE);
    }
    temp >>= 1;
  }

  PWM_On(NEC_BIT_MARK);
  PWM_Off(0);
}

void PWM_On(int ticks) {
  // Turn pin on for specified number of microseconds, then off
  DD_Reg |= _BV(DD_Bit); // Enable PWM output
  delayMicroseconds(ticks);
  DD_Reg &= ~(_BV(DD_Bit)); // Disable PWM output
}

void PWM_Off(int ticks) {
  // Leave pin off for specified number of microseconds
  delayMicroseconds(ticks);
}

void EnableIROut() {

  // WGM00-WGM02 = 101: phase-correct PWM with OCR0A as top
  // CS00 = 1: no prescaling
  TCCR0A = _BV(WGM00);
  TCCR0B = _BV(WGM02) | _BV(CS00);

  // The top value for the timer: SYSCLOCK / 2 / PWM frequency
  OCR0A = F_CPU / 2 / 38000;
  OCR0B = OCR0A / 2; // 50% duty cycle

#ifdef Tiny85
  // COM0B1 = 1: Toggle OC0B (pin 6) on compare match
  TCCR0A |= _BV(COM0B1);
#else
  // COM0A0 = 1: Toggle OC0A (pin 5) on compare match
  TCCR0A |= _BV(COM0A0);
#endif
}

// Interrupt-on-change handler
ISR (PCINT0_vect) {

  delayMicroseconds(16000); // debounce switch before reading
  delayMicroseconds(16000); // debounce switch before reading
  delayMicroseconds(16000); // debounce switch before reading

  if (digitalRead(Button1) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x45);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button1) == LOW); // Wait for button release
  }
  else if (digitalRead(Button2) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x19);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button2) == LOW); // Wait for button release
  }
  else if (digitalRead(Button3) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x16);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button3) == LOW); // Wait for button release
  }

#ifndef Tiny85
  if (digitalRead(Button4) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x44);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button4) == LOW); // Wait for button release
  }
  else if (digitalRead(Button5) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x20);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button5) == LOW); // Wait for button release
  }
  else if (digitalRead(Button6) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x17);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button6) == LOW); // Wait for button release
  }
  else if (digitalRead(Button7) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x53);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button7) == LOW); // Wait for button release
  }
  else if (digitalRead(Button8) == LOW)
  {
    digitalWrite(LED, HIGH);
    Write_IR(0x55);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    delayMicroseconds(16000);
    digitalWrite(LED, LOW);

    while (digitalRead(Button8) == LOW); // Wait for button release
  }
#endif  
}
